@@ -102,13 +102,13 @@ struct next_dim
 template <size_t I, typename TupleType>
 struct is_not_last
 {
-  static constexpr auto value = I != ndim<TupleType>::value - 1;
+  static constexpr auto value = (I != (ndim<TupleType>::value - 1));
 };
 
 template<size_t I, typename TupleType>
 struct is_last
 {
-  static constexpr auto value = I == ndim<TupleType>::value - 1;
+  static constexpr auto value = (I == (ndim<TupleType>::value - 1));
 };
 
 template <size_t I, size_t K = 0>
@@ -335,9 +335,9 @@ Iter kd_lower_bound(Iter first, Iter last, const TupleType& value)
     if (all_less(*pivot, value))
       return kd_lower_bound<J>(next(pivot), last, value);
     auto it = kd_lower_bound<J>(first, pivot, value);
-    if (none_less(*it, value)) return it;
+    if (it == last || none_less(*it, value)) return it;
     it = kd_lower_bound<J>(next(pivot), last, value);
-    if (none_less(*it, value)) return it;
+    if (it == last || none_less(*it, value)) return it;
     return last;
   }
   return none_less(*first, value) ? first : last;
@@ -355,9 +355,9 @@ Iter kd_upper_bound(Iter first, Iter last, const TupleType& value)
     if (none_less(value, *pivot))
       return kd_upper_bound<J>(next(pivot), last, value);
     auto it = kd_upper_bound<J>(first, pivot, value);
-    if (all_less(value, *it)) return it;
+    if (it == last || all_less(value, *it)) return it;
     it = kd_upper_bound<J>(next(pivot), last, value);
-    if (all_less(value, *it)) return it;
+    if (it == last || all_less(value, *it)) return it;
     return last;
   }
   return all_less(value, *first) ? first : last;
